/*
 * Hid_example_firmware.c
 *
 * Created: 22.06.2012 20:21:31
 *  Author: Flop
 */ 

#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>   /*   usbdrv.h */
#include "usbdrv.h"
#include "uart.h"

//#include "oddebug.h" /*      UART */

//    / / UART   
//  /
FILE uart_file = FDEV_SETUP_STREAM(uart_put, uart_get, _FDEV_SETUP_RW);


struct dataexchange_t       //     
{
   uchar b1;        //        3 .
   uchar b2;        //       PORTB.  
   uchar b3;        //   (  3  ).
};                  //       .
                    //        :)


struct dataexchange_t pdata = {0, 0, 0};


/*const PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = { // USB report descriptor         //       
    0x06, 0x00, 0xff,                       // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                             // USAGE (Vendor Usage 1)
    0xa1, 0x01,                             // COLLECTION (Application)
    0x15, 0x00,                             //    LOGICAL_MINIMUM (0)        // min.   
    0x26, 0xff, 0x00,                       //    LOGICAL_MAXIMUM (255)      // max.   , 255   ,     1 
    0x75, 0x08,                             //    REPORT_SIZE (8)            //   ,    "" 8 
    0x95, sizeof(struct dataexchange_t),    //    REPORT_COUNT               //   (   = 3,       )
    0x09, 0x00,                             //    USAGE (Undefined)
    0xb2, 0x02, 0x01,                       //    FEATURE (Data,Var,Abs,Buf)
    0xc0                                    // END_COLLECTION
};*/
/*      report, -     report-ID (    ).
 *     3   (  REPORT_SIZE = 8  = 1 ,   REPORT_COUNT = 3).
 */

const PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {    /* USB report descriptor */
0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
0x09, 0x01,                    // USAGE (Vendor Usage 1)
0xa1, 0x01,                    // COLLECTION (Application)
0x85, 0x77,                     //   REPORT_ID (0x77)
0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
0x75, 0x08,                    //   REPORT_SIZE (8)
0x95, 0x80,                    //   REPORT_COUNT (128)
0x09, 0x00,                    //   USAGE (Undefined)
0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
//  INPUT 
0x09, 0x01,                    //   USAGE (Vendor Usage 1)
0x95, 0x05,                     //   REPORT_COUNT (5 bytes)
0x81, 0x02,                     //   INPUT (Data,Var,Abs)

0xc0                           // END_COLLECTION
};



/*       */
static uchar    currentAddress;
static uchar    bytesRemaining;


unsigned char isInt1 = 0;


/* usbFunctionRead()        
 *    .   usbdrv.h
 */
uchar   usbFunctionRead(uchar *data, uchar len)
{
	printf_P(PSTR("usbFunctionRead()\n\r"));
    if(len > bytesRemaining)
        len = bytesRemaining;

    uchar *buffer = (uchar*)&pdata;

    if(!currentAddress)        //       .
    {                          //    
        if ( PINB & _BV(1) )
            pdata.b1 = 1;
        else
            pdata.b1 = 0;


        if ( PINB & _BV(2) )
            pdata.b2 = 1;
        else
            pdata.b2 = 0;


        if ( PINB & _BV(3) )
            pdata.b3 = 1;
        else
            pdata.b3 = 0;
    }

    uchar j;
    for(j=0; j<len; j++)
        data[j] = buffer[j+currentAddress];

    currentAddress += len;
    bytesRemaining -= len;
    return len;
}


/* usbFunctionWrite()        
 *    .   usbdrv.h
 */
uchar   usbFunctionWrite(uchar *data, uchar len)
{
	printf_P(PSTR("usbFunctionWrite()\n\r"));
    if(bytesRemaining == 0)
        return 1;               /*   */

    if(len > bytesRemaining)
        len = bytesRemaining;

    uchar *buffer = (uchar*)&pdata;
    
    uchar j;
    for(j=0; j<len; j++)
        buffer[j+currentAddress] = data[j];

    currentAddress += len;
    bytesRemaining -= len;

    if(bytesRemaining == 0)     //   
    {                           //    PORTB
        if ( pdata.b1 )
            PORTB |= _BV(1);
        else
            PORTB &= ~_BV(1);


        if ( pdata.b2 )
            PORTB |= _BV(2);
        else
            PORTB &= ~_BV(2);


        if ( pdata.b3 )
            PORTB |= _BV(3);
        else
            PORTB &= ~_BV(3);
    }

    return bytesRemaining == 0; /* 0 ,     */
}

/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
	printf_P(PSTR("usbFunctionSetup()\n\r"));
	usbRequest_t    *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* HID  */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            //      ,   report-ID
            bytesRemaining = sizeof(struct dataexchange_t);
            currentAddress = 0;
            return USB_NO_MSG;  //  usbFunctionRead()    
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            //      ,   report-ID
            bytesRemaining = sizeof(struct dataexchange_t);
            currentAddress = 0;
            return USB_NO_MSG;  //  usbFunctionWrite()     
        }
    }else{
        /*      */
    }
    return 0;
}
/* ------------------------------------------------------------------------- */


void init_devices(void)
{
	DDRB = 0b00001110;      // PB1,PB2,PB3 - 
		
	// Port D initialization
	// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In
	// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T
	PORTD=0x00;
	DDRD=0x00;
	
	// External Interrupt(s) initialization
	// INT0: Off
	// INT1: On
	// INT1 Mode: Any change
	// INT2: Off
	GICR|=0x80;
	MCUCR=0x04;
	MCUCSR=0x00;
	GIFR=0x80;
}

//   INT1
ISR(SIG_INTERRUPT1)
{
	isInt1 = 1;
}	




int main(void)
{
	init_devices();

	uart_init(); //  -    
	stdout = stdin = &uart_file; //       

	//char str[50];

    usbInit();
    usbDeviceDisconnect();  //    ,       !
    
    uchar i = 0;
    while(--i){             //  > 250 ms
        _delay_ms(1);
    }
    

    usbDeviceConnect();     // 

    sei();                  //  
	//GIMSK |= (1<<INT1);  //  INT1  

	printf_P(PSTR("Programm Start\n\r"));
	unsigned char buf[6];
	
	//for(int i; i < 10; i++)
	{
		//  
		for(int i; i < 100; i++)
		{
			usbPoll();          //        ,     - 50 ms
			_delay_ms(20);
		}
		
		_delay_ms(10);
		if (usbInterruptIsReady())
		{
			printf_P(PSTR("Send data to HID\n\r"));
			buf[0]=0x77; //ReportID
			buf[1]='K';
			buf[2]='e';
			buf[3]='y';
			buf[4]='0';
			buf[5]= 0;
			usbSetInterrupt(buf, 6);
		}
		else
		{
			printf_P(PSTR("usb Interrupt not Ready\n\r"));
		}
	}
	printf_P(PSTR("start cycle\n\r"));
	for(;;)
	{                //   
        usbPoll();          //        ,     - 50 ms
		
		if (isInt1 > 0)
		{
			printf_P(PSTR("int1\n\r"));
			_delay_ms(200);
			isInt1 = 0;
		}			
    }
    return 0;
}
/* ------------------------------------------------------------------------- */ 