/* file "uart.c" */
#include "uart.h"

typedef struct {		//   FIFO
	uint8_t	idx_w;		//     FIFO
	uint8_t	idx_r;		//     FIFO
	uint8_t	count;		//     FIFO
	uint8_t buff[UART_FIFO_SIZE]; //  FIFO
} FIFO;

static volatile
FIFO txfifo, rxfifo;

/* Initialize UART */

void uart_init()
{
	rxfifo.idx_r = 0;	
	rxfifo.idx_w = 0;	
	rxfifo.count = 0;	
	txfifo.idx_r = 0;	
	txfifo.idx_w = 0;	
	txfifo.count = 0;

	UBRRL = (F_CPU / (16UL * UART_BAUD)) - 1; //    UART
	UCSRB = _BV(RXEN)|_BV(RXCIE)|_BV(TXEN);   // : ,    , 
	/*UCSRA=0x00;
	UCSRB=0x18;
	UCSRC=0x86;
	UBRRH=0x00;
	UBRRL=0x33;*/
}

uint8_t uart_test (void)
{
	return rxfifo.count;
}

int uart_get (FILE *stream) 	// -  FIFO  
{
	char d;
	uint8_t i;

	i = rxfifo.idx_r;
	while(rxfifo.count == 0); 	//   
	d = rxfifo.buff[i++];
	cli();
	rxfifo.count--;
	sei();
	if(i >= UART_FIFO_SIZE)
		i = 0;
	rxfifo.idx_r = i;
	uart_put(d,stream); 		//  
	return d;
}

int uart_put (char d, FILE *stream) // -   FIFO  
{
	uint8_t i;

	i = txfifo.idx_w;
	while(txfifo.count >= UART_FIFO_SIZE); //      FIFO
	txfifo.buff[i++] = d;
	cli();
	txfifo.count++;
	UCSRB |= _BV(UDRIE); //       
	sei();
	if(i >= UART_FIFO_SIZE)
		i = 0;
	txfifo.idx_w = i;
	return 0;
}

ISR(USART_RXC_vect) //   
{
	uint8_t d, n, i;

	d = UDR;
	n = rxfifo.count;
	if(n < UART_FIFO_SIZE) //  FIFO    
	{
		rxfifo.count = ++n;
		i = rxfifo.idx_w;
		rxfifo.buff[i++] = d;
		if(i >= UART_FIFO_SIZE)
			i = 0;
		rxfifo.idx_w = i;
	}
}

ISR(USART_UDRE_vect) //     
{
	uint8_t n, i;

	n = txfifo.count;
	if(n) {
		txfifo.count = --n;
		i = txfifo.idx_r;
		UDR = txfifo.buff[i++];
		if(i >= UART_FIFO_SIZE)
			i = 0;
		txfifo.idx_r = i;
	}
	if(n == 0) //  FIFO ,
		UCSRB &= ~_BV(UDRIE); //        
}
